/*
 * Copyright (c) 2000, 2001
 *       Yes/No Software (Yoshiyuki Nakano). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/bios.h>

#include "main.h"
#include "vscreen.h"
#include "gate.h"

#include "gateu.bmh"
#include "gated.bmh"

#define GATE_MAX 64

static unsigned short work[8];

/* Q[g̏oԊu, LN^P */
static int gate_interval = 7;

/* pbg12Ԃgp */
#define PALETTE 12
static unsigned short palette = ((PALETTE-8)<<9);

/* Aj[Vp1hbgÂ炵p^[C`16 */
#define GATE_CHARSET (128+16)

/* [\pXvCg̃Oobt@Ǘϐ */
/* ő吔̃LN^ƍő吔*2̃XvCg */
/* LN^: 128`CXvCg: 96` */
#define GATE_SP_MAX 16
#define GATE_SP_CHAR 128
#define GATE_SP_SPRITE 0
static int gate_sp_next;

#define GATE_STATE_UNUSED 0
#define GATE_STATE_WAIT   1
#define GATE_STATE_ANIM   2
#define GATE_STATE_READY  3
#define GATE_STATE_CLEAR  4

static struct gate {
	int state;    /*  0: gp 1: ҋ@ 2: LтĂr 3: Lт */
	int sleep;    /* WAITԂANIMԂɈڍs܂ł̎ */
	int x;        
	int u, d;     /* ㋫EʒuCEʒu(㉺̃hbg) */
	int au, ad;   /* Aj[VpCǂ܂ŐLт(㉺̃hbg) */
	int au8, ad8;   /* ̃o8{x(...) */
	int spu, spd; /* [hbgpXvCg */
} gate[GATE_MAX];

void
gate_init()
{
	int i;
	
	for (i = 0; i < GATE_MAX; i++) {
		gate[i].state = GATE_STATE_UNUSED;
	}
	
	gate_sp_next = 0;
	
	/* 炵p^[o^ */
	for (i = 0; i < 8; i++) {
		font_set_colordata(GATE_CHARSET + i    , 1, bmp_gateu + i);
		font_set_colordata(GATE_CHARSET + i + 8, 1, bmp_gateu + i);
	}
}

/*
 * [hbg\̂߂̃XvCgmۂ
 */
int
gate_get_sp()
{
	int ret = gate_sp_next;
	gate_sp_next = (gate_sp_next + 1) % GATE_SP_MAX;
	return ret;
}

/*
 * Q[gLт̊Jn
 */
void
gate_start(struct gate *g)
{
	g->state = GATE_STATE_ANIM;
	/* Aj[Vpo */
	g->au = 0;
	g->ad = 0;
	g->au8 = 0;
	g->ad8 = 0;
	/* XvCgm */
	g->spu = gate_get_sp();
	g->spd = gate_get_sp();
}

/*
 * 1Xe[W̃Q[g쐬
 * starthbgCendhbg܂
 * Q[ǧԂlevelhbg
 * ́Cseedŏ
 */
void
gate_create(int start, int end, int level, int seed)
{
	int i, j, k;
	
	srand(seed);
	
	for (i = start&~7, j = 0; i < end && j < GATE_MAX;
			i += gate_interval*8, j++) {
		gate[j].state = GATE_STATE_WAIT;
		gate[j].sleep = 0;
		gate[j].x = i;
		k = rand()%(GAME_HEIGHT*8-level);
		gate[j].u = k;
		gate[j].d = GAME_HEIGHT*8 - k - level;
	}
	
	/* ŏɈĂɏoĂ̂}邽 */
	gate[0].sleep = 0;
	gate[1].sleep = 50;
	gate[2].sleep = 100;
	gate[3].sleep = 150;
}

void
gate_release(struct gate *g)
{
	g->state = GATE_STATE_UNUSED;
}

/*
 * eQ[g1t[vZ\
 */
void
gate_a_frame(struct gate *g)
{
	int sp, ch, i, j;

	switch (g->state) {
	case GATE_STATE_UNUSED:
		break;
	case GATE_STATE_WAIT:
		if (g->sleep) {
			g->sleep--;
		}
		if (!g->sleep &&
			vscreen_get_scrollx() + LCD_PIXEL_WIDTH - 8 >= g->x) {
			gate_start(g);
		}
		break;
	case GATE_STATE_ANIM:
		/** 㑤XV **/
		g->au8 += (g->u*8 - g->au8)/48 + 1;
		if (g->au8 > g->u*8) {
			g->au8 = g->u*8;
		}
		g->au = g->au8>>3;
		
		/** XV **/
		g->ad8 += (g->d*8 - g->ad8)/48 + 1;
		if (g->ad8 > g->d*8) {
			g->ad8 = g->d*8;
		}
		g->ad = g->ad8>>3;
		
		/*** 㑤\ ***/
		/** p^[쐬 **/
		ch = GATE_SP_CHAR + g->spu;
		sp = GATE_SP_SPRITE + g->spu*2;
		
		
		/* XvCg */
		i = g->au%8;
		j = g->u%8;
		memcpy(work, &bmp_gateu[(j-i)&7], i*2);
		memset(work+i, 0, 16-i*2);
		font_set_colordata(ch, 1, work);
		
		/** LN^\ **/
		i = (j-i)&7;
		vscreen_fill_char_c((g->x&255)>>3, GAME_Y, 2, g->au>>3, 
							GATE_CHARSET + i);
		/*
		for (j = 7; j < g->au; j+=8) {
			vscreen_set_char(g->x  , GAME_Y*8 + j, GATE_CHARSET + i);
			vscreen_set_char(g->x+8, GAME_Y*8 + j, GATE_CHARSET + i);
		}
		*/
		
		/*** \ ***/
		/** p^[쐬 **/
		ch = GATE_SP_CHAR + g->spd;
		sp = GATE_SP_SPRITE + g->spd*2;
		
		/* XvCg */
		i = g->ad%8;
		j = g->d%8;
		memset(work, 0, 16);
		memcpy(work+8-i, &bmp_gateu[8-j], i*2);
		font_set_colordata(ch, 1, work);
		
		/** LN^\ **/
		i = (i-j)&7;
		vscreen_fill_char_c((g->x&255)>>3, GAME_Y+GAME_HEIGHT-(g->ad>>3),
							2, g->ad>>3, GATE_CHARSET+8+i);
		/*
		for (j = 8; j <= g->ad; j+=8) {
			vscreen_set_char(g->x  , GAME_Y*8+GAME_HEIGHT*8 - j,
				GATE_CHARSET+8+i);
			vscreen_set_char(g->x+8, GAME_Y*8+GAME_HEIGHT*8 - j,
				GATE_CHARSET+8+i);
		}
		*/
		/***  ***/
		if (g->ad == g->d && g->au == g->u) {
			g->state = GATE_STATE_READY;
		}
		
		/*** ̂܂܃XvCg\邽߂Ɂ ***/
		/* continuing... */
	case GATE_STATE_READY:
	case GATE_STATE_CLEAR:
		/* XvCg\ */
		/* 㑤 */
		ch = GATE_SP_CHAR + g->spu;
		sp = GATE_SP_SPRITE + g->spu*2;
		vsprite_set_char_location(sp, 
				g->x,
				(GAME_Y*8 + g->au)&~7,
				ch|palette);
		vsprite_set_char_location(sp + 1, 
				g->x + 8,
				(GAME_Y*8 + g->au)&~7,
				ch|palette);
		
		/*  */
		ch = GATE_SP_CHAR + g->spd;
		sp = GATE_SP_SPRITE + g->spd*2;
		vsprite_set_char_location(sp,
				g->x,
				(GAME_Y*8 + GAME_HEIGHT*8 - g->ad)&~7,
				ch|palette);
		vsprite_set_char_location(sp + 1,
				g->x + 8,
				(GAME_Y*8 + GAME_HEIGHT*8 - g->ad)&~7,
				ch|palette);
				
		/* ʂOꂽH */
		if (g->x + 16 < vscreen_get_scrollx()) {
			gate_release(g);
		}
		break;
	default:
		break;
	}
}

/*
 * 1t[vZ&\
 */
void
gate_frame()
{
	int i;
	struct gate *g = gate;
	
	for (i = 0; i < GATE_MAX; i++, g++) {
		gate_a_frame(g);
	}
}

/*
 * 蔻
 * -1: 
 * 0 : ĂȂ
 * other: ߂ɂQ[gID
 */
int
gate_collision(int x, int y, int w)
{
	int i;
	struct gate *g = gate;
	
	if (y < GAME_Y*8 || y >= GAME_Y*8 + GAME_HEIGHT*8) {
		return -1;
	}
	
	for (i = 0; i < GATE_MAX; i++, g++) {
		if (g->state == GATE_STATE_UNUSED) {
			continue;
		}
		
		if (x >= g->x - w && x < g->x + w + 16) {
			/* قƂɓĂ邩 */
			if (x >= g->x && x < g->x + 16) {
				/*  */
				if (y >= GAME_Y*8 && y < GAME_Y*8 + g->au) {
					return -1;
				}
				/*  */
				if (y < GAME_Y*8 + GAME_HEIGHT*8 &&
					y >= GAME_Y*8 + GAME_HEIGHT*8 - g->ad) {
					return -1;
				}
			}
			
			/* ĂȂǋ߂ */
			return i+1;
		}
	}
	
	return 0;
}

/*
 * Q[gɓĂ邩
 */
int
gate_a_collision(int id, int x, int y)
{
	struct gate *g = &gate[id-1];

	if (x >= g->x && x < g->x + 16) {
		/*  */
		if (y >= GAME_Y*8 && y < GAME_Y*8 + g->au) {
			return GAME_Y*8 + g->au;
		}
		/*  */
		if (y <  GAME_Y*8 + GAME_HEIGHT*8 &&
			y >= GAME_Y*8 + GAME_HEIGHT*8 - g->ad) {
			return GAME_Y*8 + GAME_HEIGHT*8 - g->ad;
		}
	}
	return 0;
}
